@@ -1,5 +1,11 @@
 """Module for loading config."""
 
+try:
+    from importlib.resources import files
+except ImportError:  # pragma: no cover
+    # fallback for python <=3.8
+    from importlib_resources import files  # type: ignore
+
 import logging
 import os
 import os.path
@@ -9,13 +15,29 @@ from dataclasses import dataclass
 
 import pluggy
 from itertools import chain
-from typing import Dict, Iterator, List, Tuple, Any, Optional, Union, Iterable, Callable
+from typing import (
+    TYPE_CHECKING,
+    Dict,
+    Iterator,
+    List,
+    Set,
+    Tuple,
+    Any,
+    Optional,
+    Type,
+    Union,
+    Iterable,
+    Callable,
+)
 from pathlib import Path
 from sqlfluff.core.plugin.host import get_plugin_manager
 from sqlfluff.core.errors import SQLFluffUserError
 
 import appdirs
 
+if TYPE_CHECKING:  # pragma: no cover
+    from sqlfluff.core.templaters.base import RawTemplater
+
 if sys.version_info >= (3, 11):
     import tomllib
 else:  # pragma: no cover
@@ -184,7 +206,7 @@ def coerce_value(val: str) -> Any:
     return v
 
 
-def nested_combine(*dicts: dict) -> dict:
+def nested_combine(*dicts: Dict[str, Any]) -> Dict[str, Any]:
     """Combine an iterable of dictionaries.
 
     Each dictionary is combined into a result dictionary. For
@@ -208,7 +230,7 @@ def nested_combine(*dicts: dict) -> dict:
     >>> nested_combine({"a": {"b": "c"}}, {"a": {"b": "e"}})
     {'a': {'b': 'e'}}
     """
-    r: dict = {}
+    r: Dict[str, Any] = {}
     for d in dicts:
         for k in d:
             if k in r and isinstance(r[k], dict):
@@ -224,8 +246,10 @@ def nested_combine(*dicts: dict) -> dict:
     return r
 
 
-def dict_diff(left: dict, right: dict, ignore: Optional[List[str]] = None) -> dict:
-    """Work out the difference between to dictionaries.
+def dict_diff(
+    left: Dict[str, Any], right: Dict[str, Any], ignore: Optional[List[str]] = None
+) -> Dict[str, Any]:
+    """Work out the difference between two dictionaries.
 
     Returns a dictionary which represents elements in the `left`
     dictionary which aren't in the `right` or are different to
@@ -256,7 +280,7 @@ def dict_diff(left: dict, right: dict, ignore: Optional[List[str]] = None) -> di
     >>> dict_diff({"a": "b"}, {"a": "c"}, ["a"])
     {}
     """
-    buff: dict = {}
+    buff: Dict[str, Any] = {}
     for k in left:
         if ignore and k in ignore:
             continue
@@ -290,6 +314,32 @@ def split_comma_separated_string(raw: Union[str, List[str]]) -> List[str]:
     )
 
 
+def split_colon_separated_string(in_str: str) -> Tuple[Tuple[str, ...], str]:
+    """Converts a colon separated string.
+
+    NOTE: This also includes some provisions for values which may be
+    Windows paths containing colons and NOT stripping those.
+    """
+    config_path: List[str] = []
+    for element in in_str.split(":"):
+        # If the next element begins with a backslash, and the previous
+        # one had length == 1,  then this is probably a windows path.
+        # In which case, rejoin them together.
+        element = element.strip()
+        if (
+            element
+            and element[0] == "\\"
+            and config_path[-1]
+            and len(config_path[-1]) == 1
+        ):
+            config_path[-1] = config_path[-1] + ":" + element
+        else:
+            # Otherwise just add it to the path.
+            config_path.append(element)
+
+    return tuple(config_path[:-1]), config_path[-1]
+
+
 class ConfigLoader:
     """The class for loading config files.
 
@@ -302,7 +352,7 @@ class ConfigLoader:
 
     def __init__(self) -> None:
         # TODO: check that this cache implementation is actually useful
-        self._config_cache: dict = {}
+        self._config_cache: Dict[str, Dict[str, Any]] = {}
 
     @classmethod
     def get_global(cls) -> "ConfigLoader":
@@ -362,7 +412,9 @@ class ConfigLoader:
         return buff
 
     @classmethod
-    def _iter_config_elems_from_dict(cls, configs: dict) -> Iterator[ConfigElemType]:
+    def _iter_config_elems_from_dict(
+        cls, configs: Dict[str, Any]
+    ) -> Iterator[ConfigElemType]:
         """Walk a config dict and get config elements.
 
         >>> list(
@@ -380,7 +432,7 @@ class ConfigLoader:
                 yield (key,), val
 
     @classmethod
-    def _config_elems_to_dict(cls, configs: Iterable[ConfigElemType]) -> dict:
+    def _config_elems_to_dict(cls, configs: Iterable[ConfigElemType]) -> Dict[str, Any]:
         """Reconstruct config elements into a dict.
 
         >>> ConfigLoader._config_elems_to_dict(
@@ -388,13 +440,13 @@ class ConfigLoader:
         ... )
         {'foo': {'bar': {'baz': 'a', 'biz': 'b'}}}
         """
-        result: Dict[str, Union[dict, str]] = {}
+        result: Dict[str, Any] = {}
         for key, val in configs:
             ref = result
             for step in key[:-1]:
                 if step not in ref:
                     ref[step] = {}
-                ref = ref[step]  # type: ignore
+                ref = ref[step]
             ref[key[-1]] = val
         return result
 
@@ -412,7 +464,9 @@ class ConfigLoader:
         return cls._walk_toml(tool)
 
     @classmethod
-    def _get_config_elems_from_file(cls, fpath: str) -> List[ConfigElemType]:
+    def _get_config_elems_from_file(
+        cls, fpath: Optional[str] = None, config_string: Optional[str] = None
+    ) -> List[ConfigElemType]:
         """Load a config from a file and return a list of tuples.
 
         The return value is a list of tuples, were each tuple has two elements,
@@ -428,19 +482,27 @@ class ConfigLoader:
             string value will remain.
 
         """
-        buff: List[Tuple[tuple, Any]] = []
+        assert fpath or config_string, "One of fpath or config_string is required."
+        buff: List[ConfigElemType] = []
         # Disable interpolation so we can load macros
-        kw: Dict = {}
+        kw: Dict[str, Any] = {}
         kw["interpolation"] = None
         config = configparser.ConfigParser(delimiters="=", **kw)
         # NB: We want to be case sensitive in how we read from files,
         # because jinja is also case sensitive. To do this we override
         # the optionxform attribute.
         config.optionxform = lambda option: option  # type: ignore
-        config.read(fpath)
+        if fpath:
+            config.read(fpath)
+        else:
+            assert config_string
+            config.read_string(config_string)
+            # Set the fpath to the current working directory
+            fpath = os.getcwd()
+
         for k in config.sections():
             if k == "sqlfluff":
-                key: Tuple = ("core",)
+                key: Tuple[str, ...] = ("core",)
             elif k.startswith("sqlfluff:"):
                 # Return a tuple of nested values
                 key = tuple(k[len("sqlfluff:") :].split(":"))
@@ -470,7 +532,7 @@ class ConfigLoader:
         return buff
 
     @classmethod
-    def _resolve_path(cls, fpath, val):
+    def _resolve_path(cls, fpath: str, val: str) -> str:
         """Try to resolve a path."""
         # Make the referenced path.
         ref_path = os.path.join(os.path.dirname(fpath), val)
@@ -478,7 +540,9 @@ class ConfigLoader:
         return ref_path if os.path.exists(ref_path) else val
 
     @staticmethod
-    def _incorporate_vals(ctx: dict, vals: List[ConfigElemType]) -> dict:
+    def _incorporate_vals(
+        ctx: Dict[str, Any], vals: List[ConfigElemType]
+    ) -> Dict[str, Any]:
         """Take a list of tuples and incorporate it into a dictionary.
 
         >>> ConfigLoader._incorporate_vals({}, [(("a", "b"), "c")])
@@ -509,7 +573,7 @@ class ConfigLoader:
 
     @staticmethod
     def _validate_configs(
-        configs: Iterable[ConfigElemType], file_path
+        configs: Iterable[ConfigElemType], file_path: str
     ) -> List[ConfigElemType]:
         """Validate config elements.
 
@@ -599,10 +663,30 @@ class ConfigLoader:
             validated_configs.append((k, v))
         return validated_configs
 
+    def load_config_resource(
+        self, package: str, file_name: str, configs: Optional[Dict[str, Any]] = None
+    ) -> Dict[str, Any]:
+        """Load a config resource.
+
+        This is however more compatible with mypyc because it avoids
+        the use of the __file__ object to find the default config.
+
+        This is only tested extensively with the default config.
+
+        NOTE: This requires that the config file is built into
+        a package but should be more performant because it leverages
+        importlib.
+        https://docs.python.org/3/library/importlib.resources.html
+        """
+        config_string = files(package).joinpath(file_name).read_text()
+        elems = self._get_config_elems_from_file(config_string=config_string)
+        elems = self._validate_configs(elems, package + "." + file_name)
+        return self._incorporate_vals(configs or {}, elems)
+
     def load_config_file(
-        self, file_dir: str, file_name: str, configs: Optional[dict] = None
-    ) -> dict:
-        """Load the default config file."""
+        self, file_dir: str, file_name: str, configs: Optional[Dict[str, Any]] = None
+    ) -> Dict[str, Any]:
+        """Load a config file."""
         file_path = os.path.join(file_dir, file_name)
         if file_name == "pyproject.toml":
             elems = self._get_config_elems_from_toml(file_path)
@@ -611,7 +695,15 @@ class ConfigLoader:
         elems = self._validate_configs(elems, file_path)
         return self._incorporate_vals(configs or {}, elems)
 
-    def load_config_at_path(self, path: str) -> dict:
+    def load_config_string(
+        self, config_string: str, configs: Optional[Dict[str, Any]] = None
+    ) -> Dict[str, Any]:
+        """Load a config from the string in cfg format."""
+        elems = self._get_config_elems_from_file(config_string=config_string)
+        elems = self._validate_configs(elems, "<config string>")
+        return self._incorporate_vals(configs or {}, elems)
+
+    def load_config_at_path(self, path: str) -> Dict[str, Any]:
         """Load config from a given path."""
         # First check the cache
         if str(path) in self._config_cache:
@@ -627,7 +719,7 @@ class ConfigLoader:
             "pyproject.toml",
         ]
 
-        configs: dict = {}
+        configs: Dict[str, Any] = {}
 
         if os.path.isdir(path):
             p = path
@@ -644,7 +736,7 @@ class ConfigLoader:
         self._config_cache[str(path)] = configs
         return configs
 
-    def load_extra_config(self, extra_config_path: str) -> dict:
+    def load_extra_config(self, extra_config_path: str) -> Dict[str, Any]:
         """Load specified extra config."""
         if not os.path.exists(extra_config_path):
             raise SQLFluffUserError(
@@ -655,7 +747,7 @@ class ConfigLoader:
         if str(extra_config_path) in self._config_cache:
             return self._config_cache[str(extra_config_path)]
 
-        configs: dict = {}
+        configs: Dict[str, Any] = {}
         if extra_config_path.endswith("pyproject.toml"):
             elems = self._get_config_elems_from_toml(extra_config_path)
         else:
@@ -684,7 +776,7 @@ class ConfigLoader:
 
         return user_config_dir_path
 
-    def load_user_appdir_config(self) -> dict:
+    def load_user_appdir_config(self) -> Dict[str, Any]:
         """Load the config from the user's OS specific appdir config directory."""
         user_config_dir_path = self._get_user_config_dir_path()
         if os.path.exists(user_config_dir_path):
@@ -692,7 +784,7 @@ class ConfigLoader:
         else:
             return {}
 
-    def load_user_config(self) -> dict:
+    def load_user_config(self) -> Dict[str, Any]:
         """Load the config from the user's home directory."""
         user_home_path = os.path.expanduser("~")
         return self.load_config_at_path(user_home_path)
@@ -702,7 +794,7 @@ class ConfigLoader:
         path: str,
         extra_config_path: Optional[str] = None,
         ignore_local_config: bool = False,
-    ) -> dict:
+    ) -> Dict[str, Any]:
         """Loads a selection of config files from both the path and its parent paths."""
         user_appdir_config = (
             self.load_user_appdir_config() if not ignore_local_config else {}
@@ -727,8 +819,11 @@ class ConfigLoader:
 
     @classmethod
     def find_ignore_config_files(
-        cls, path, working_path=Path.cwd(), ignore_file_name=".sqlfluffignore"
-    ):
+        cls,
+        path: str,
+        working_path: Union[str, Path] = Path.cwd(),
+        ignore_file_name: str = ".sqlfluffignore",
+    ) -> Set[str]:
         """Finds sqlfluff ignore files from both the path and its parent paths."""
         return set(
             filter(
@@ -743,7 +838,9 @@ class ConfigLoader:
         )
 
     @staticmethod
-    def iter_config_locations_up_to_path(path, working_path=Path.cwd()):
+    def iter_config_locations_up_to_path(
+        path: str, working_path: Union[str, Path] = Path.cwd()
+    ) -> Iterator[str]:
         """Finds config locations from both the path and its parent paths.
 
         The lowest priority is the user appdir, then home dir, then increasingly
@@ -782,10 +879,10 @@ class FluffConfig:
 
     def __init__(
         self,
-        configs: Optional[dict] = None,
+        configs: Optional[Dict[str, Any]] = None,
         extra_config_path: Optional[str] = None,
         ignore_local_config: bool = False,
-        overrides: Optional[dict] = None,
+        overrides: Optional[Dict[str, Any]] = None,
         plugin_manager: Optional[pluggy.PluginManager] = None,
         # Ideally a dialect should be set when config is read but sometimes
         # it might only be set in nested .sqlfluff config files, so allow it
@@ -887,7 +984,7 @@ class FluffConfig:
                 f"{', '.join([d.label for d in dialect_readout()])}"
             )
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         # Copy the object's state from self.__dict__ which contains
         # all our instance attributes. Always use the dict.copy()
         # method to avoid modifying the original state.
@@ -900,7 +997,7 @@ class FluffConfig:
         state["_configs"]["core"].pop("templater_obj", None)
         return state
 
-    def __setstate__(self, state):  # pragma: no cover
+    def __setstate__(self, state: Dict[str, Any]) -> None:  # pragma: no cover
         # Restore instance attributes
         self.__dict__.update(state)
         # NB: We don't reinstate the plugin manager, but this should only
@@ -922,8 +1019,8 @@ class FluffConfig:
         cls,
         extra_config_path: Optional[str] = None,
         ignore_local_config: bool = False,
-        overrides: Optional[dict] = None,
-        **kw,
+        overrides: Optional[Dict[str, Any]] = None,
+        **kw: Any,
     ) -> "FluffConfig":
         """Loads a config object just based on the root directory."""
         loader = ConfigLoader.get_global()
@@ -940,13 +1037,33 @@ class FluffConfig:
             **kw,
         )
 
+    @classmethod
+    def from_string(
+        cls,
+        config_string: str,
+        extra_config_path: Optional[str] = None,
+        ignore_local_config: bool = False,
+        overrides: Optional[Dict[str, Any]] = None,
+        plugin_manager: Optional[pluggy.PluginManager] = None,
+    ) -> "FluffConfig":
+        """Loads a config object given a particular path."""
+        loader = ConfigLoader.get_global()
+        c = loader.load_config_string(config_string)
+        return cls(
+            configs=c,
+            extra_config_path=extra_config_path,
+            ignore_local_config=ignore_local_config,
+            overrides=overrides,
+            plugin_manager=plugin_manager,
+        )
+
     @classmethod
     def from_path(
         cls,
         path: str,
         extra_config_path: Optional[str] = None,
         ignore_local_config: bool = False,
-        overrides: Optional[dict] = None,
+        overrides: Optional[Dict[str, Any]] = None,
         plugin_manager: Optional[pluggy.PluginManager] = None,
     ) -> "FluffConfig":
         """Loads a config object given a particular path."""
@@ -999,9 +1116,11 @@ class FluffConfig:
 
         return cls(overrides=overrides, require_dialect=require_dialect)
 
-    def get_templater(self, templater_name="jinja", **kwargs):
+    def get_templater(
+        self, templater_name: str = "jinja", **kwargs: Any
+    ) -> "RawTemplater":
         """Fetch a templater by name."""
-        templater_lookup = {
+        templater_lookup: Dict[str, Type["RawTemplater"]] = {
             templater.name: templater
             for templater in chain.from_iterable(
                 self._plugin_manager.hook.get_templaters()
@@ -1033,7 +1152,7 @@ class FluffConfig:
             plugin_manager=self._plugin_manager,
         )
 
-    def diff_to(self, other: "FluffConfig") -> dict:
+    def diff_to(self, other: "FluffConfig") -> Dict[str, Any]:
         """Compare this config to another.
 
         Args:
@@ -1052,7 +1171,7 @@ class FluffConfig:
 
     def get(
         self, val: str, section: Union[str, Iterable[str]] = "core", default: Any = None
-    ):
+    ) -> Any:
         """Get a particular value from the config."""
         section_dict = self.get_section(section)
         if section_dict is None:
@@ -1085,7 +1204,7 @@ class FluffConfig:
                     return None
             return buff
 
-    def set_value(self, config_path: Iterable[str], val: Any):
+    def set_value(self, config_path: Iterable[str], val: Any) -> None:
         """Set a value at a given path."""
         # Make the path a list so we can index on it
         config_path = list(config_path)
@@ -1097,7 +1216,7 @@ class FluffConfig:
         # Current section:
         dict_buff = [self._configs]
         for elem in config_path[:-1]:
-            dict_buff.append(dict_buff[-1][elem])
+            dict_buff.append(dict_buff[-1].get(elem, {}))
         # Set the value
         dict_buff[-1][config_path[-1]] = config_val
         # Rebuild the config
@@ -1106,7 +1225,9 @@ class FluffConfig:
             dict_buff[-1][elem] = dict_elem
         self._configs = dict_buff[0]
 
-    def iter_vals(self, cfg: Optional[dict] = None) -> Iterable[tuple]:
+    def iter_vals(
+        self, cfg: Optional[Dict[str, Any]] = None
+    ) -> Iterable[Tuple[Any, ...]]:
         """Return an iterable of tuples representing keys.
 
         We show values before dicts, the tuple contains an indent
@@ -1135,7 +1256,7 @@ class FluffConfig:
                 for idnt, key, val in self.iter_vals(cfg=cfg[k]):
                     yield (idnt + 1, key, val)
 
-    def process_inline_config(self, config_line: str, fname: str):
+    def process_inline_config(self, config_line: str, fname: str) -> None:
         """Process an inline config command and update self."""
         # Strip preceding comment marks
         if config_line.startswith("--"):
@@ -1147,18 +1268,18 @@ class FluffConfig:
             )
             return
         config_line = config_line[9:].strip()
-        # Divide on colons
-        config_path = [elem.strip() for elem in config_line.split(":")]
-        config_val = (tuple(config_path[:-1]), config_path[-1])
+        config_val = split_colon_separated_string(config_line)
         # Validate the value
         ConfigLoader._validate_configs([config_val], fname)
         # Set the value
         self.set_value(*config_val)
-        # If the config is for dialect, initialise the dialect
-        if config_path[:-1] == ["dialect"]:
-            self._initialise_dialect(config_path[-1])
+        # If the config is for dialect, initialise the dialect.
+        # NOTE: Comparison with a 1-tuple is intentional here as
+        # the first element of config_val is a tuple.
+        if config_val[0] == ("dialect",):
+            self._initialise_dialect(config_val[1])
 
-    def process_raw_file_for_config(self, raw_str: str, fname: str):
+    def process_raw_file_for_config(self, raw_str: str, fname: str) -> None:
         """Process a full raw file for inline config and update self."""
         # Scan the raw file for config commands.
         for raw_line in raw_str.splitlines():
