@@ -1,5 +1,20 @@
-"""Errors - these are closely linked to what used to be called violations."""
-from typing import Optional, Tuple, Any, List
+"""Errors - these are closely linked to what used to be called violations.
+
+NOTE: The BaseException class, which ValueError inherits from, defines
+a custom __reduce__() method for picking and unpickling exceptions.
+For the SQLBaseError, and it's dependent classes, we define properties
+of these exceptions which don't work well with that method, which is
+why we redefine __reduce__() on each of these classes. Given the
+circumstances in which they are called, they don't show up on coverage
+tracking.
+
+https://stackoverflow.com/questions/49715881/how-to-pickle-inherited-exceptions
+"""
+from typing import Optional, Tuple, Any, List, Dict, Type, Union, TYPE_CHECKING
+
+if TYPE_CHECKING:  # pragma: no cover
+    from sqlfluff.core.parser import PositionMarker, BaseSegment
+    from sqlfluff.core.rules import BaseRule, LintFix
 
 CheckTuple = Tuple[str, int, int]
 
@@ -9,71 +24,66 @@ class SQLBaseError(ValueError):
 
     _code: Optional[str] = None
     _identifier = "base"
+    _warning = False  # The default value for `warning`
 
     def __init__(
         self,
-        *args,
-        pos=None,
-        line_no=0,
-        line_pos=0,
-        ignore=False,
-        fatal=False,
-        warning=False,
-        **kwargs
-    ):
+        description: Optional[str] = None,
+        pos: Optional["PositionMarker"] = None,
+        line_no: int = 0,
+        line_pos: int = 0,
+        ignore: bool = False,
+        fatal: bool = False,
+        warning: Optional[bool] = None,
+    ) -> None:
         self.fatal = fatal
         self.ignore = ignore
-        self.warning = warning
+        self.warning: bool = warning if warning is not None else self._warning
+        self.description = description
         if pos:
             self.line_no, self.line_pos = pos.source_position()
         else:
             self.line_no = line_no
             self.line_pos = line_pos
-        super().__init__(*args, **kwargs)
+        super().__init__(self.desc())
+
+    def __eq__(self, other) -> bool:
+        """Errors compare equal if they are the same type and same content."""
+        if not isinstance(other, self.__class__):
+            return False
+        return self.__dict__ == other.__dict__
+
+    def __reduce__(
+        self,
+    ) -> Tuple[Type["SQLBaseError"], Tuple[Any, ...]]:
+        """Prepare the SQLBaseError for pickling."""
+        return type(self), (
+            self.description,
+            None,
+            self.line_no,
+            self.line_pos,
+            self.ignore,
+            self.fatal,
+            self.warning,
+        )
 
     @property
-    def fixable(self):
+    def fixable(self) -> bool:
         """Should this error be considered fixable?"""
         return False
 
     def rule_code(self) -> str:
-        """Fetch the code of the rule which cause this error.
-
-        NB: This only returns a real code for some subclasses of
-        error, (the ones with a `rule` attribute), but otherwise
-        returns a placeholder value which can be used instead.
-        """
-        if hasattr(self, "rule"):
-            return getattr(self, "rule").code
-
+        """Fetch the code of the rule which cause this error."""
         return self._code or "????"
 
     def desc(self) -> str:
-        """Fetch a description of this violation.
-
-        NB: For violations which don't directly implement a rule
-        this attempts to return the error message linked to whatever
-        caused the violation. Optionally some errors may have their
-        description set directly.
-        """
-        if hasattr(self, "description") and getattr(self, "description", None):
-            # This can only override if it's present AND
-            # if it's non-null.
-            return getattr(self, "description")
-
-        if hasattr(self, "rule"):
-            return getattr(self, "rule").description
-
-        # Return the first element - probably a string message
-        if len(self.args) > 1 and isinstance(self.args, str):  # pragma: no cover TODO?
-            return self.args
-
-        if len(self.args) == 1:
-            return self.args[0]
+        """Fetch a description of this violation."""
+        if self.description:
+            return self.description
 
         return self.__class__.__name__  # pragma: no cover
 
-    def get_info_dict(self):
+    def get_info_dict(self) -> Dict[str, Union[str, int]]:
         """Return a dict of properties.
 
         This is useful in the API for outputting violations.
@@ -98,12 +108,12 @@ class SQLBaseError(ValueError):
         """Return hashable source signature for deduplication."""
         return (self.check_tuple(), self.desc())
 
-    def ignore_if_in(self, ignore_iterable: List[str]):
+    def ignore_if_in(self, ignore_iterable: List[str]) -> None:
         """Ignore this violation if it matches the iterable."""
         if self._identifier in ignore_iterable:
             self.ignore = True
 
-    def warning_if_in(self, warning_iterable: List[str]):
+    def warning_if_in(self, warning_iterable: List[str]) -> None:
         """Warning only for this violation if it matches the iterable.
 
         Designed for rule codes so works with L001, LL0X but also TMP or PRS
@@ -159,12 +169,41 @@ class SQLParseError(SQLBaseError):
     _code = "PRS"
     _identifier = "parsing"
 
-    def __init__(self, *args, segment=None, **kwargs):
+    def __init__(
+        self,
+        description: Optional[str] = None,
+        segment: Optional["BaseSegment"] = None,
+        line_no: int = 0,
+        line_pos: int = 0,
+        ignore: bool = False,
+        fatal: bool = False,
+        warning: Optional[bool] = None,
+    ) -> None:
         # Store the segment on creation - we might need it later
         self.segment = segment
-        if self.segment:
-            kwargs["pos"] = self.segment.pos_marker
-        super().__init__(*args, **kwargs)
+        super().__init__(
+            description=description,
+            pos=segment.pos_marker if segment else None,
+            line_no=line_no,
+            line_pos=line_pos,
+            ignore=ignore,
+            fatal=fatal,
+            warning=warning,
+        )
+
+    def __reduce__(
+        self,
+    ) -> Tuple[Type["SQLParseError"], Tuple[Any, ...]]:
+        """Prepare the SQLParseError for pickling."""
+        return type(self), (
+            self.description,
+            self.segment,
+            self.line_no,
+            self.line_pos,
+            self.ignore,
+            self.fatal,
+            self.warning,
+        )
 
 
 class SQLLintError(SQLBaseError):
@@ -184,24 +223,51 @@ class SQLLintError(SQLBaseError):
     _identifier = "linting"
 
     def __init__(
-        self, *args, segment=None, rule=None, fixes=None, description=None, **kwargs
-    ):
-        # Something about position, message and fix?
+        self,
+        description: str,
+        segment: "BaseSegment",
+        rule: "BaseRule",
+        fixes: Optional[List["LintFix"]] = None,
+        ignore: bool = False,
+        fatal: bool = False,
+        warning: Optional[bool] = None,
+    ) -> None:
         self.segment = segment
-        if self.segment:
-            kwargs["pos"] = self.segment.pos_marker
         self.rule = rule
         self.fixes = fixes or []
-        self.description = description
-        super().__init__(*args, **kwargs)
+        super().__init__(
+            description=description,
+            pos=segment.pos_marker if segment else None,
+            ignore=ignore,
+            fatal=fatal,
+            warning=warning,
+        )
+
+    def __reduce__(
+        self,
+    ) -> Tuple[Type["SQLLintError"], Tuple[Any, ...]]:
+        """Prepare the SQLLintError for pickling."""
+        return type(self), (
+            self.description,
+            self.segment,
+            self.rule,
+            self.fixes,
+            self.ignore,
+            self.fatal,
+            self.warning,
+        )
 
     @property
-    def fixable(self):
+    def fixable(self) -> bool:
         """Should this error be considered fixable?"""
         if self.fixes:
             return True
         return False
 
+    def rule_code(self) -> str:
+        """Fetch the code of the rule which cause this error."""
+        return self.rule.code
+
     def source_signature(self) -> Tuple[Any, ...]:
         """Return hashable source signature for deduplication.
 
@@ -218,7 +284,7 @@ class SQLLintError(SQLBaseError):
         )
         return (self.check_tuple(), self.description, fix_raws, source_fixes)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return "<SQLLintError: rule {} pos:{!r}, #fixes: {}, description: {}>".format(
             self.rule_code(),
             (self.line_no, self.line_pos),
@@ -227,5 +293,13 @@ class SQLLintError(SQLBaseError):
         )
 
 
+class SQLUnusedNoQaWarning(SQLBaseError):
+    """A warning about an unused noqa directive."""
+
+    _code = "NOQA"
+    _identifier = "noqa"
+    _warning = True
+
+
 class SQLFluffUserError(ValueError):
     """An error which should be fed back to the user."""
