@@ -47,7 +47,7 @@ class LintingResult:
         return {key: d1.get(key, 0) + d2.get(key, 0) for key in keys}
 
     @staticmethod
-    def combine_dicts(*d: dict) -> dict:
+    def combine_dicts(*d: Dict[str, Any]) -> Dict[str, Any]:
         """Take any set of dictionaries and combine them."""
         dict_buffer: dict = {}
         for dct in d:
@@ -58,7 +58,7 @@ class LintingResult:
         """Add a new `LintedDir` to this result."""
         self.paths.append(path)
 
-    def stop_timer(self):
+    def stop_timer(self) -> None:
         """Stop the linting timer."""
         self.total_time = time.monotonic() - self._start_time
 
@@ -107,14 +107,14 @@ class LintingResult:
         """Count the number of violations in the result."""
         return sum(path.num_violations(**kwargs) for path in self.paths)
 
-    def get_violations(self, **kwargs):
+    def get_violations(self, **kwargs) -> list:
         """Return a list of violations in the result."""
         buff = []
         for path in self.paths:
             buff += path.get_violations(**kwargs)
         return buff
 
-    def violation_dict(self, **kwargs):
+    def violation_dict(self, **kwargs) -> dict:
         """Return a dict of paths and violations."""
         return self.combine_dicts(
             *(path.violation_dict(**kwargs) for path in self.paths)
