@@ -25,7 +25,10 @@ from sqlfluff.core.parser.segments.meta import (
     MetaSegment,
     Indent,
     Dedent,
+    ImplicitIndent,
     TemplateSegment,
+    EndOfFile,
+    TemplateLoop,
 )
 
 __all__ = (
@@ -47,7 +50,10 @@ __all__ = (
     "MetaSegment",
     "Indent",
     "Dedent",
+    "ImplicitIndent",
     "TemplateSegment",
+    "EndOfFile",
+    "TemplateLoop",
     "IdentitySet",
     "FixPatch",
     "SourceFix",
