@@ -6,10 +6,9 @@ This is a newer slicing algorithm that handles cases heuristic.py does not.
 from dataclasses import dataclass, field
 import logging
 import regex
-from typing import Callable, cast, Dict, List, NamedTuple, Optional, Tuple
+from typing import Callable, cast, Dict, List, NamedTuple, Optional, Tuple, Union
 
 from jinja2 import Environment
-from jinja2.environment import Template
 from jinja2.exceptions import TemplateSyntaxError
 
 from sqlfluff.core.templaters.base import (
@@ -52,14 +51,14 @@ class JinjaTracer:
         raw_sliced: List[RawFileSlice],
         raw_slice_info: Dict[RawFileSlice, RawSliceInfo],
         sliced_file: List[TemplatedFileSlice],
-        make_template: Callable[[str], Template],
+        render_func: Callable[[str], str],
     ):
         # Input
         self.raw_str = raw_str
         self.raw_sliced = raw_sliced
         self.raw_slice_info = raw_slice_info
         self.sliced_file = sliced_file
-        self.make_template = make_template
+        self.render_func = render_func
 
         # Internal bookkeeping
         self.program_counter: int = 0
@@ -73,8 +72,7 @@ class JinjaTracer:
             else rs.raw
             for rs in self.raw_sliced
         )
-        trace_template = self.make_template(trace_template_str)
-        trace_template_output = trace_template.render()
+        trace_template_output = self.render_func(trace_template_str)
         # Split output by section. Each section has two possible formats.
         trace_entries: List[regex.Match] = list(
             regex.finditer(r"\0", trace_template_output)
@@ -127,10 +125,10 @@ class JinjaTracer:
         # 'append_to_templated' gets the default value of "", empty string.)
         # For more detail, see the comments near the call to slice_file() in
         # plugins/sqlfluff-templater-dbt/sqlfluff_templater_dbt/templater.py.
-        templated_str = self.make_template(self.raw_str).render() + append_to_templated
+        templated_str = self.render_func(self.raw_str) + append_to_templated
         return JinjaTrace(templated_str, self.raw_sliced, self.sliced_file)
 
-    def find_slice_index(self, slice_identifier) -> int:
+    def find_slice_index(self, slice_identifier: Union[int, str]) -> int:
         """Given a slice identifier, return its index.
 
         A slice identifier is a string like 00000000000000000000000000000002.
@@ -146,7 +144,9 @@ class JinjaTracer:
             )
         return raw_slices_search_result[0]
 
-    def move_to_slice(self, target_slice_idx, target_slice_length):
+    def move_to_slice(
+        self, target_slice_idx: int, target_slice_length: Union[int, str]
+    ) -> None:
         """Given a template location, walk execution to that point."""
         while self.program_counter < len(self.raw_sliced):
             self.record_trace(
@@ -174,7 +174,9 @@ class JinjaTracer:
                 candidates.sort(key=lambda c: abs(target_slice_idx - c))
                 self.program_counter = candidates[0]
 
-    def record_trace(self, target_slice_length, slice_idx=None, slice_type=None):
+    def record_trace(
+        self, target_slice_length, slice_idx=None, slice_type=None
+    ) -> None:
         """Add the specified (default: current) location to the trace."""
         if slice_idx is None:
             slice_idx = self.program_counter
@@ -202,7 +204,7 @@ class JinjaAnalyzer:
     re_open_tag = regex.compile(r"^\s*({[{%])[\+\-]?\s*")
     re_close_tag = regex.compile(r"\s*[\+\-]?([}%]})\s*$")
 
-    def __init__(self, raw_str: str, env: Environment):
+    def __init__(self, raw_str: str, env: Environment) -> None:
         # Input
         self.raw_str: str = raw_str
         self.env = env
@@ -337,7 +339,7 @@ class JinjaAnalyzer:
         "raw_begin": "block",
     }
 
-    def analyze(self, make_template: Callable[[str], Template]) -> JinjaTracer:
+    def analyze(self, render_func: Callable[[str], str]) -> JinjaTracer:
         """Slice template in jinja."""
         # str_buff and str_parts are two ways we keep track of tokens received
         # from Jinja. str_buff concatenates them together, while str_parts
@@ -473,7 +475,7 @@ class JinjaAnalyzer:
             self.raw_sliced,
             self.raw_slice_info,
             self.sliced_file,
-            make_template,
+            render_func,
         )
 
     def track_templated(
@@ -493,7 +495,7 @@ class JinjaAnalyzer:
 
     def track_call(
         self, m_open: regex.Match, m_close: regex.Match, tag_contents: List[str]
-    ):
+    ) -> RawSliceInfo:
         """Set up tracking for "{% call ... %}"."""
         unique_alternate_id = self.next_slice_id()
         open_ = m_open.group(1)
@@ -524,7 +526,9 @@ class JinjaAnalyzer:
         self.idx_raw += len(raw)
 
     @staticmethod
-    def extract_block_type(tag_name, block_subtype):
+    def extract_block_type(
+        tag_name: str, block_subtype: Optional[str] = None
+    ) -> Tuple[str, Optional[str]]:
         """Determine block type."""
         # :TRICKY: Syntactically, the Jinja {% include %} directive looks like
         # a block, but its behavior is basically syntactic sugar for
