@@ -227,6 +227,7 @@ UNRESERVED_KEYWORDS = [
     "ARITHABORT",
     "ARITHIGNORE",
     "AT",
+    "AUTO_CREATE_TABLE",
     "AUTO",
     "BERNOULLI",
     "BINARY",
@@ -244,6 +245,7 @@ UNRESERVED_KEYWORDS = [
     "COMMITTED",
     "COMPRESS_ALL_ROW_GROUPS",
     "COMPRESSION_DELAY",
+    "COMPRESSION",
     "CONCAT_NULL_YIELDS_NULL",
     "CONCAT",
     "CONNECTION_OPTIONS",
@@ -251,6 +253,7 @@ UNRESERVED_KEYWORDS = [
     "CONTINUE",
     "CONTROL",
     "CREDENTIAL",
+    "COPY",
     "CURSOR_CLOSE_ON_COMMIT",
     "CYCLE",
     "DATA_COMPRESSION",
@@ -281,6 +284,7 @@ UNRESERVED_KEYWORDS = [
     "ENCRYPTED",
     "ENCRYPTION_TYPE",
     "ENCRYPTION",
+    "ERRORFILE_CREDENTIAL",
     "ERRORFILE_DATA_SOURCE",
     "ERRORFILE",
     "EXPAND",
@@ -295,6 +299,7 @@ UNRESERVED_KEYWORDS = [
     "FILESTREAM",
     "FILESTREAM_ON",
     "FILESTREAM",
+    "FILE_TYPE",
     "FILETABLE_COLLATE_FILENAME",
     "FILETABLE_DIRECTORY",
     "FILETABLE_FULLPATH_UNIQUE_CONSTRAINT_NAME",
